import re

class FileWrapper:
    """Wrapper to convert file-like objects to iterables"""

    def __init__(self, filelike, blksize=8192):
        self.filelike = filelike
        self.blksize = blksize
        if hasattr(filelike,'close'):
            self.close = filelike.close

    def __getitem__(self,key):
        data = self.filelike.read(self.blksize)
        if data:
            return data
        raise IndexError

    def __iter__(self):
        return self

    def __next__(self):
        data = self.filelike.read(self.blksize)
        if data:
            return data
        raise StopIteration

def csvtodict(f, sep):
    res = {}
    for l in f.readlines():
        l = re.split(sep, l.strip())
        addr = ()

        foo = l[0].strip()
        if ':' in foo:
            ip, port = foo.split(':') 
            addr = (ip, int(port))
        else:
            addr = (foo, 8080)
        res[addr] = tuple(l[1:])
    return res

if __name__ == '__main__':
    res = {}
    with open('test.txt', 'r') as f:
        res.update(csvtodict(f, '[,，]'))
    print(res)




'''
    Process Manager
    A wrapper of psutil with some useful functions
'''
import re
import warnings
import sys
import signal
from psutil import *

from .linux import lxrun
from .exception import mute


Process.cmd = lambda self: ' '.join(self.cmdline())


@mute(not __debug__, [])
def getpids(pattern, spec='cmd'):
    '''get pids list by command'''
    foo = "ps -eo pid,{0} | \
           grep -v -P '(sudo|grep|PID.*COMMAND)' | \
           grep -P '{1}'".format(spec, pattern)
    foo = lxrun(foo).split('\n')
    
    p = re.compile(' *(\d+)')
    return [int(p.match(i).group(1)) for i in foo if p.match(i)]
    

def kill(pid):
    _, err = lxrun('kill -9 {0}'.format(pid), err=True)
    if err:
        return False
    return True

def kills(cmd):
    res = 0
    for p in getpids(cmd):
        if kill(p):
            res += 1
    return res

# --------Useful wrapper based on psutil------------
def pid_of_listen_port(port): 
    tcps = net_connections(kind='tcp') 
    for conn in tcps:
        if conn.laddr.port == port and conn.status == CONN_LISTEN:
            return conn.pid
    return 0


def kill_proc_tree(p, sig=signal.SIGTERM, include_parent=True,
      timeout=None, on_terminate=None):
    """Kill a process tree (including grandchildren) with signal
    "sig" and return a (gone, still_alive) tuple.
    "on_terminate", if specified, is a callabck function which is
    called as soon as a child terminates.
    """
    if isinstance(p, Process):
        pid = p.pid
        parent = p
    else:
        pid = p
        parent = Process(p)
    if pid == os.getpid():
        raise RuntimeError("I refuse to kill myself")
    parent = Process(pid)
    children = parent.children(recursive=True)
    if include_parent:
        children.append(parent)
    for p in children:
        p.send_signal(sig)
    gone, alive = wait_procs(children, timeout=timeout,
                                    callback=on_terminate)
    return (gone, alive)


def reap_children(timeout=3):
    """Tries hard to terminate and ultimately kill all the children of this process.
    This may be useful in unit tests whenever sub-processes are started. 
    This will help ensure that no extra children (zombies) stick around to hog resources.
    """
    def on_terminate(proc):
        print("process {} terminated with exit code {}".format(proc, proc.returncode))

    procs = Process().children()
    # send SIGTERM
    for p in procs:
        p.terminate()
    gone, alive = wait_procs(procs, timeout=timeout, callback=on_terminate)
    if alive:
        # send SIGKILL
        for p in alive:
            print("process {} survived SIGTERM; trying SIGKILL" % p)
            p.kill()
        gone, alive = wait_procs(alive, timeout=timeout, callback=on_terminate)
        if alive:
            # give up
            for p in alive:
                print("process {} survived SIGKILL; giving up" % p)
# --------------------------------------------------



def getpid(cmd=None, pattern=None):
    if cmd is not None:
        foo = getpids(cmd)
        if foo:
            return foo[0]
        return 0

    if pattern is not None:
        pattern = '^.*(:[0-9]{{2}}){{2}} (?!sudo).*{}.*'.format(pattern)
        cmd = "ps -ef | grep -P '{0}'".format(pattern)
        res = lxrun(cmd)
        res = res.strip()
        if not res:
            return 0
        else:
            res = res.split('\n')
            res = filter(lambda x: 'grep' not in x, res)
            res = list(res)
            pid = re.split(' +', res[0])[1]
            return int(pid)

def getports(command=''):
    return get_port_names(command).keys()

def get_port_names(command='', pid=-1):
    '''get tcp listening port and names'''
    foo = lxrun("netstat -plnt")
    foo = foo.split('\n')
    
    p = 'tcp +\d+ +\d+ +[^ ]*:(\d+) .+' + \
        (str(pid) if pid>=0 else '\d+') + '/' + \
        '([^ ]*{0}[^ ]*)'.format(command)

    p = re.compile(p)
    res = {}
    for i in foo:
        m = p.match(i)
        if m:
            k, v= m.groups()
            if re.search(command, v):
                k = int(k)
                res[k] = v.strip()
    return res


def reboot(pid):
    cmd = get_command(pid)
    kill(pid)
    lxrun(cmd, daemon=True)
    pid = get_pid(pattern=cmd)
    return pid


# ----------------- Full tested internal function  -----------------------


# Old version API
def getcmd(pid):
    warnings.warn("pm.getcmd(pid) will be deleted in the futur.\nUse pm.get(pid, 'cmd') instead.", DeprecationWarning)
    print(sys._getframe().f_back.f_globals['__file__'], ': line ', sys._getframe().f_back.f_lineno)
    return get(pid, 'cmd')




def mute(cond, default=None):
    def dec(func):
        if cond:
            def f(*args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except:
                    return default
            return f
            
        else:
            return func
    return dec



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

setup(
    name='AIJIdevtools',
    version='0.3.0',
    author='AIJI',
    author_email='thecrazyaiji@gmail.com',
    description='Some useful helper-funcs for devpers',
    packages=['devtools'],
    install_requires=['psutil'],
    url='https://github.com/AIJIJI/devtools'
                                                                                )



'''
    About tomcat manager
'''
import os
from .pm import *

DEFAULTPATH = '/usr/local/services'


def find_tomcat():
    _, dirs, _ = os.walk(DEFAULTPATH)
    for d in dirs:
        if 'tomcat' in d:
            yield d

class Tomcat:
    
    def __init__(self, rootdir=DEFAULTPATH, name='tomcat', port=8080):
        self.basedir = os.path.join(rootdir, name)
        self.name = name
        self.port = port

    def getpid(self):
        pids = getpids(self.name)
        if len(pids) >= 1:
            return pids[0]
        return 0
        
    def getproc(self):
        p = self.getpid()
        if p:
            return Process(p)
        else:
            return None
   
    def status(self):
        proc = self.getproc()
        if not proc:
            dic = {'status': 'dead'}
            return dic 
        net_dic = {}
        dic = {}
        
        conns = net_connections('tcp')
        conns_local = [i for i in conns if i.laddr[1] == self.port]
        conn_types = {key: value for key, value in globals().items() if key.startswith('CONN_')}
        for key, value in conn_types.items():
            foo = [i for i in conns_local if i.status == value]
            net_dic[key[5:]] = len(foo)
            
            

        with proc.oneshot():

            if [i for i in conns if i.status == CONN_LISTEN and i.laddr[1] == self.port]:
                dic['status'] = 'open'
            else:
                dic['status'] = 'closed'

            foo = proc.memory_info().rss
            dic['drs'] = '%.2fMB' % (foo / (1024 * 1024))
            dic['cpu'] = '%s%%' % proc.cpu_percent()
        dic['conns'] = net_dic
        return dic

    def stop(self):
        if self.getpid():
            foo = os.path.join(self.basedir, 'bin/shutdown.sh')
            lxrun(['bash', foo])

    def start(self):
        foo = os.path.join(self.basedir, 'bin/startup.sh')
        lxrun(['bash', foo])
    
    def restart(self):
        self.stop()
        for i in range(10):
            if self.getpid():
                time.sleep(1)
            else:
                break
        self.start()





# coding: utf8
from subprocess import Popen, PIPE, DEVNULL
import os
import re
import psutil

from .format import tostr
from .exception import mute


__all__ = ['lxrun', 'lxget', 'SPEC']



class Spec:
    # if the specifier is less than zero,
    # then the meth:lxget need no extra argument.
    COMMAND = 2             # COMMAND
    PID = 1
    MANUFACTURER = -1       
    OS_VERSION = -2         
    PRODUCT_NAME = -3
    SERIAL_NUMBER = -4
    IO_WAIT = -5            # time waiting for I/O completion
    MAC_ADDRESS = -6
    HOSTNAME = -7

    def base(self):
        l = list(filter(lambda x: x[0].isupper() and getattr(self, x) < 0, dir(self)))
        return l

SPEC = Spec()


@mute(not __debug__, '')
def lxget(spec, pid=None, err=False, **kwargs):
    res = ''
    errmsg = ''
    # SWITCH
    if spec == Spec.IO_WAIT:
        foo, errmsg = lxrun('top -b -n 1 | grep wa', err=True)
        p = re.compile(' *([0-9.]*) *wa')
        m = p.search(foo)
        res =  m.group(1)

    elif spec == Spec.MANUFACTURER:
        res, errmsg = lxrun("dmidecode -s system-manufacturer", err=True)
    
    elif spec == Spec.OS_VERSION:
        res, errmsg = lxrun("sed -n '1,1p' /etc/issue", err=True)

    elif spec == Spec.PRODUCT_NAME:
        res, errmsg = lxrun("dmidecode -s system-product-name", err=True)
        
    elif spec == Spec.SERIAL_NUMBER:
        res, errmsg = lxrun("dmidecode -s system-serial-number", err=True)
        if res == 'Not Specified' or res.startswith('VM'):
            res = ''

    elif spec == Spec.MAC_ADDRESS:
        res, errmsg = lxrun("ifconfig -a", err=True)
        p = ':'.join(['(\w{2})']*6)
        res = ''.join(re.search(p, res).groups())

    elif spec == Spec.HOSTNAME:
        res = lxrun('hostname')


    else:
        pass
    
    # RETURN
    if err:
        return res, errmsg
    else:
        return res
        



def lxrun(cmd, err=False, bg=False, daemon=False):
    if isinstance(cmd, list):
        cmd = ' '.join(cmd)

    if bg or daemon:
        p = Popen(cmd, shell=True, stdout=DEVNULL if daemon else None)
        return p 
        
    res = None
    p =  Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = tostr(stdout).rstrip()
    stderr = tostr(stderr).rstrip()
    if err:
        return stdout, stderr
    else:
        return stdout


def pyrun(cmd, err=False, daemon=False):
    p = _py3_gen().__next__()
    cmd = ' '.join([p, cmd])
    return lxrun(cmd, err=err, daemon=daemon)

def _py3_gen():
    names = ['python3', 'python'] + ['python3.' + str(i) for i in range(8)]
    p = re.compile('is (/.*python3.*)|is aliased to \'(/.*python3.*)\'')
    for n in names:
        foo, err = lxrun('type ' + n, err=True)
        m = p.search(foo)
        if m:
            yield m.group(1)

    




# coding: utf8
import re
def tostr(sth):
    if isinstance(sth, str):
        return sth
    elif isinstance(sth, bytes):
        return sth.decode()
    else:
        return str(sth)

def tobytes(sth):
    if isinstance(sth, bytes):
        return sth
    elif isinstance(sth, str):
        return sth.encode
    else:
        return bytes(sth)


def toaddr(strs):
    host = 'localhost'
    port = 0
    if isinstance(strs, list):
        if len(strs) == 1:
            return toaddr(strs[0])
        elif len(strs) == 2:
            host = strs[0]
            port = int(strs[1])
        else:
            raise ValueError
        return (host, port)
    elif isinstance(strs, str):
        strs = strs.split(':')
        if len(strs) == 1:
            host = strs[0]
            return (host, port)
        elif len(strs) == 2:
            return toaddr(strs)
        else:
            raise ValueError
    else:
        raise ValueError

def strtodict(text, mode, *args):
    '''
    # parse text to dict
    # mode: '[rc][v]'
    #     site 0: keys' orientation, (r)ow or (c)olumn
    #     site 1: multi (v)lues, return a list of dict 
    # mode c: len(args) == 1
    #     k1: v1
    #     k2: v2
    # mode cs: len(args) == 2
    #     k1: d1v1, d2v1
    #     k2: d1v2, d2v2, d3v2,
    # mode r(s): len(args) == 1
    #     k1   k2    k3 
    #     v1   v2    v3 
    #     d2v1 d2v2  d3v3
    
    '''
    if mode == 'c':
        rows = filter(lambda x: x.strip(), re.split('\n', text))
        dic = {}
        sep = args[0]
        for r in rows:
            try:
                k, v = re.split(sep, r, 1)
            except Exception as why:
                continue
            else:
                k, v = k.strip(), v.strip()
                dic[k] = v
        return dic
    
    elif mode == 'cs':
        dics = []
        dic = strtodict(text, 'c', args[0])
        for (k, v) in dic.items():
            values = [i.strip() for i in re.split(args[1], v) if i.strip()]
            while len(values) > len(dics):
                dics.append({})
            for n, value in enumerate(values):
                dics[n][k] = value
        return dics
    
    elif mode == 'rs':
        dics = []
        items = list(filter(lambda x: x.strip(), re.split('\n', text)))
        sep = args[0]
        keys = re.split(sep, items[0])
        for values in items[1:]:
            dics.append(dict(zip(keys, re.split(sep, values))))
        return dics
    elif mode == 'r':
        return strtodict(text, 'rs', *args)[0]


                
        

